@@ -43,6 +43,9 @@
 #include <dclib/cconfig.h>
 #include <dclib/csearchmanager.h>
 
+// need to decompress zline data
+#include <dclib/core/czlib.h>
+
 #include "cclient.h"
 
 CClient::CClient()
@@ -71,11 +74,17 @@ CClient::CClient()
 	m_nReconnectCount      = 0;
 	m_timeReconnectTimeout = 0;
 	m_timeMyinfoTimeout    = 0;
+	
+	// zline handling
+	m_zBuffer.SetSize(0);
+	m_bZMode = FALSE;
 }
 
 CClient::~CClient()
 {
 	SetCallBackFunction(0);
+	
+	m_IPcache.Clear();
 }
 
 /** */
@@ -101,6 +110,12 @@ void CClient::ConnectionState( eConnectionState state )
 		m_nOpListHandler    = 0;
 
 		m_UserList.Clear();
+		
+		// hub IP may have changed
+		m_IPcache.Clear();
+		
+		m_zBuffer.SetSize(0);
+		m_bZMode = FALSE;
 #ifdef HAVE_SSL
 		m_pMessageSSL.Init();
 #endif
@@ -172,7 +187,7 @@ void CClient::DataAvailable( const char * buffer, int len )
 	i = 0;
 
 	s = m_sBuffer + CString().Set(buffer,len);
-
+	
 	// search last '|'
 	i = s.FindRev('|');
 	i++;
@@ -190,6 +205,13 @@ void CClient::DataAvailable( const char * buffer, int len )
 	else
 	{
 		m_sBuffer = s.Mid( i, s.Length()-i );
+		
+		// need to process zline data
+		if ( m_bZMode == TRUE )
+		{
+			HandleMessage( m_sBuffer );
+			m_sBuffer = "";
+		}
 	}
 }
 
@@ -298,6 +320,7 @@ int CClient::HandleMessage( const CString & message )
 	CMessageHandler MessageHandler;
 	CString sAnswer,s;
 	CEncrypt Encrypt;
+	bool foundZOn = FALSE;
 
 	if ( message == "" )
 	{
@@ -306,6 +329,41 @@ int CClient::HandleMessage( const CString & message )
 
 	pointer = 0;
 
+	if ( m_bZMode == TRUE )
+	{
+		m_zBuffer.Append( message.Data(), message.Length() );
+		
+		CByteArray * inflated = new CByteArray();
+		CZLib * pZlib = new CZLib();
+		
+		unsigned long size = 0;
+		
+		size = pZlib->OneShotInflate( (const char*) m_zBuffer.Data(), m_zBuffer.Size(), inflated );
+		
+		if ( size == 0 )
+		{
+			// not an error - may be waiting for more data
+			//printf("CClient::HandleMessage: Failed to inflate zline data!\n");
+		}
+		else
+		{
+			//printf("CClient::HandleMessage: Inflated %ld to %ld\n", m_zBuffer.Size(), size);
+			CString mes;
+			mes.Set((const char*) inflated->Data(), size);
+			//printf("Inflated=%s\n", mes.Data());
+			
+			m_bZMode = FALSE;
+			m_zBuffer.SetSize(0);
+			
+			HandleMessage(mes);
+		}
+		
+		delete inflated;
+		delete pZlib;
+		
+		return 0;
+	}
+
 	while( (type=MessageHandler.Parse(&message,pointer,Object=0)) != DC_MESSAGE_PARSE_ERROR )
 	{
 		if ( !Object )
@@ -342,6 +400,31 @@ int CClient::HandleMessage( const CString & message )
 						// No $GetINFO
 						s += "NoGetINFO ";
 
+						if ( CConfig::Instance() != 0 )
+						{
+							if ( CConfig::Instance()->GetDisableHashList() == FALSE )
+							{
+								s += "TTHSearch ";
+							}
+							
+							if ( CConfig::Instance()->GetUserIP2Enabled() == TRUE )
+							{
+								s += "UserIP2 ";
+							}
+							
+							if ( CConfig::Instance()->GetZLineEnabled() == TRUE )
+							{
+								// this is what DC++ 0.699 sends
+								// it may change later after testing finishes
+								s += "ZPipe0 ";
+							}
+							
+							if ( CConfig::Instance()->GetCompressedTransfers() == TRUE )
+							{
+								s += "GetZBlock ";
+							}
+						}
+
 						if ( s != "" )
 						{
 							SendSupports(s);
@@ -526,7 +609,30 @@ int CClient::HandleMessage( const CString & message )
 
 					if ( (search == TRUE) && (CQueryManager::Instance() != 0) )
 					{
-						CQueryManager::Instance()->SearchQuery( GetHubName(), GetHost(), GetNick(), msg );
+						// there may be a lot of search requests
+						// avoid looking up the same IP address each time
+						// so search for IP in cache
+						CString * ipaddr = 0;
+						if ( m_IPcache.Get( GetIP(), (CObject*&)ipaddr ) == 0 )
+						{
+							// found in cache;
+							//printf("Found in cache=%s\n", ipaddr->Data() );
+						}
+						else
+						{
+							// not found in cache
+							ipaddr = new CString();
+							*ipaddr = GetHostByName(GetIP());
+							
+							// add to cache
+							m_IPcache.Add( GetIP(), ipaddr );
+							
+							//printf("Looked up=%s\n", ipaddr->Data() );
+						}
+						
+						CString ipwithport = *ipaddr + ":" + CString().setNum(GetPort());
+						
+						CQueryManager::Instance()->SearchQuery( GetHubName(), ipwithport, GetNick(), msg );
 					}
 				}
 
@@ -535,13 +641,20 @@ int CClient::HandleMessage( const CString & message )
 
 			case DC_MESSAGE_SEARCHRESULT:
 			{
+				// fix hub name
+				CMessageSearchResult * msg = (CMessageSearchResult *) Object;
+				if ( msg->m_sHubName == msg->m_sHubHost )
+				{
+					msg->m_sHubName = GetHubName();
+				}
+				
 				if( m_bHandleUserList == TRUE )
 				{
-					UpdateUserSlots( (CMessageSearchResult *) Object );
+					UpdateUserSlots( msg );
 				}
 
 				if ( CSearchManager::Instance() )
-					if ( CSearchManager::Instance()->HandleSearch( (CMessageSearchResult *) Object ) == TRUE )
+					if ( CSearchManager::Instance()->HandleSearch( msg ) == TRUE )
 						Object = 0;
 				break;
 			}
@@ -629,6 +742,36 @@ int CClient::HandleMessage( const CString & message )
 				break;
 			}
 			
+			case DC_MESSAGE_USERIP:
+			{
+				CMessageUserIP * msg = (CMessageUserIP*) Object;
+				
+				if ( (CConfig::Instance() != 0) && (CConfig::Instance()->GetUserIP2Enabled() == TRUE) )
+				{
+					if ( (GetMode() == ecmACTIVE) && (GetNick() == msg->m_sNick) )
+					{
+						// if using external IP and not network interface
+						if ( CConfig::Instance()->GetExternalIP() == TRUE )
+						{
+							CConfig::Instance()->SetHost(msg->m_sIP);
+						}
+					}
+				}
+				
+				break;
+			}
+			
+			case DC_MESSAGE_ZLINE:
+			{
+				foundZOn = TRUE;
+				
+				// nothing else needs to handle the object
+				delete Object;
+				Object = 0;
+				
+				break;
+			}
+			
 			default:
 			{
 				break;
@@ -653,6 +796,14 @@ int CClient::HandleMessage( const CString & message )
 				delete Object;
 			}
 		}
+		
+		if ( foundZOn == TRUE )
+		{
+			m_bZMode = TRUE;
+			HandleMessage(message.Mid(pointer, message.Length() - pointer));
+			
+			break;
+		}
 	}
 
 	return 0;
