@@ -30,6 +30,9 @@
 #include <dclib/cclientssl.h>
 #include <dclib/cuserlist.h>
 
+// added for zline data buffer
+#include <dclib/core/cbytearray.h>
+
 /** */
 typedef enum eReconnectState {
 	ersNONE,
@@ -105,6 +108,8 @@ public:
 	/** */
 	inline void SetUsedPassword( bool b );
 	
+	/** Returns true if supports UserIP2 */
+	inline bool GetSupportsUserIP2() const;
 	/** */
 	inline bool IsHandshake() const;
 	
@@ -212,6 +217,12 @@ private:
 	CMessageCapabilities m_MessageCapabilities;
 	/** */
 	CMessageSupports m_MessageSupports;
+	/** Cache of IP addresses for handling searches */
+	CStringList m_IPcache;
+	/** Store of ZLine compressed data */
+	CByteArray m_zBuffer;
+	/** Are we receiving zline data */
+	bool m_bZMode;
 };
 
 /** inline functions */
@@ -310,5 +321,8 @@ inline void CClient::SetHandleTransfer( bool b )
 /** */
 inline void CClient::SetSSLMode( bool b )
 { m_ClientThread.Lock(); m_bSSLMode = b; m_ClientThread.UnLock(); }
+/** */
+inline bool CClient::GetSupportsUserIP2() const
+{ return m_MessageSupports.m_bUserIP2; }
 
 #endif
