@@ -162,6 +162,7 @@
 
 #define XML_DISABLEHASHLIST		"disablehashlist"
 #define XML_COMPRESSEDTRANSFERS		"enablecompressedtransfers"
+#define XML_ENABLE_ZLINE		"enable_zline"
 
 #define XML_TRAFFIC_RX			"trafficrx"
 #define XML_TRAFFIC_TX			"traffictx"
@@ -177,6 +178,10 @@
 
 #define XML_REMOTE_ENCODING		"remote_encoding"
 
+#define XML_USERIP2_ENABLED		"userip2_enabled"
+
+#define XML_READ_FILE_BUFFER_SIZE	"readfilebuffersize"
+
 #ifdef WIN32
 #define DCGUI_CONFIG_PATH	"\\dc"
 #else
@@ -386,6 +391,19 @@ CConfig::CConfig( CString configpath )
 	/** Default to ISO-8859-1 (previous behaviour) */
 	m_sRemoteEncoding = "ISO-8859-1";
 	
+	/** Default to UserIP2 support disabled */
+	m_bUserIP2Enabled = FALSE;
+	
+	/** ZLine support default disabled */
+	m_bEnableZLine = FALSE;
+	
+	/** Used for calculating session transfer stats */
+	m_nStartDownloaded = 0;
+	m_nStartUploaded = 0;
+	
+	/** Default to 40KiB */
+	m_nReadFileBufferSize = 40960;
+	
 	SetInstance(this);
 }
 
@@ -575,6 +593,7 @@ int CConfig::SaveDCLib()
 	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_LISTENHOST, m_sListenHost );	
 	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_SENDMESSAGEONACTIVEMODEREQUEST, bSendMessageOnActiveModeRequest );
 	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_CHECKPRIVATEADDRESSSPACE, bCheckPrivateAddressSpace );
+	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_USERIP2_ENABLED, m_bUserIP2Enabled );
 
 	// hublisturl
 	node = xmlNewChild( doc->children, 0, (const xmlChar*)XML_HUBLISTURL, 0 );
@@ -603,10 +622,12 @@ int CConfig::SaveDCLib()
 	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_FORCEMOVEENABLED, bForceMoveEnabled );
 	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_DISABLEHASHLIST, m_bDisableHashList );
 	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_COMPRESSEDTRANSFERS, m_bCompressedTransfers );
+	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_ENABLE_ZLINE, m_bEnableZLine );
 	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_AUTOSEARCHINTERVAL, CString().setNum(m_nAutoSearchInterval) );
 	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_SMALLFILESIZE, CString().setNum(m_nSmallFileSize) );
 	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_DONTSHAREDOTFILES, m_bDontShareDotFiles );
 	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_REMOTE_ENCODING, m_sRemoteEncoding );
+	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_READ_FILE_BUFFER_SIZE, CString().setNum(m_nReadFileBufferSize) );
 
 	// search history
 	node = xmlNewChild( doc->children, 0, (const xmlChar*)XML_SEARCH_HISTORY, 0 );
@@ -1444,9 +1465,15 @@ void CConfig::ParseDCLibConfig( xmlNodePtr node )
 				else if ( xml_name2 == XML_HUBOFFLINE_TRANSFERCLOSE )
 					eHubOfflineTransferClose = (eCloseType)xml->content(n2).asINT();
 				else if ( xml_name2 == XML_TRAFFIC_RX )
+				{
 					CSocket::m_Traffic.AddTraffic( ettRX, xml->content(n2).asULL() );
+					m_nStartDownloaded = xml->content(n2).asULL();
+				}
 				else if ( xml_name2 == XML_TRAFFIC_TX )
+				{
 					CSocket::m_Traffic.AddTraffic( ettTX, xml->content(n2).asULL() );
+					m_nStartUploaded = xml->content(n2).asULL();
+				}
 				else if ( xml_name2 == XML_TRAFFIC_DATA_RX )
 					CSocket::m_Traffic.AddTraffic( ettDATARX, xml->content(n2).asULL() );
 				else if ( xml_name2 == XML_TRAFFIC_DATA_TX )
@@ -1485,6 +1512,8 @@ void CConfig::ParseDCLibConfig( xmlNodePtr node )
 					bSendMessageOnActiveModeRequest = xml->getBoolChild(n2);
 				else if ( xml_name2 == XML_CHECKPRIVATEADDRESSSPACE )
 					bCheckPrivateAddressSpace = xml->getBoolChild(n2);
+				else if ( xml_name2 == XML_USERIP2_ENABLED )
+					m_bUserIP2Enabled = xml->getBoolChild(n2);
 			}
 		}
 		/* other entrys */
@@ -1518,6 +1547,8 @@ void CConfig::ParseDCLibConfig( xmlNodePtr node )
 					m_bDisableHashList = xml->getBoolChild(n2);
 				else if ( xml_name2 == XML_COMPRESSEDTRANSFERS )
 					m_bCompressedTransfers = xml->getBoolChild(n2);
+				else if ( xml_name2 == XML_ENABLE_ZLINE )
+					m_bEnableZLine = xml->getBoolChild(n2);
 				else if ( xml_name2 == XML_AUTOSEARCHINTERVAL )
 					m_nAutoSearchInterval = xml->content(n2).asLONG();
 				else if ( xml_name2 == XML_SMALLFILESIZE )
@@ -1526,6 +1557,8 @@ void CConfig::ParseDCLibConfig( xmlNodePtr node )
 					m_bDontShareDotFiles = xml->getBoolChild(n2);
 				else if ( xml_name2 == XML_REMOTE_ENCODING )
 					m_sRemoteEncoding = xml->content(n2);
+				else if ( xml_name2 == XML_READ_FILE_BUFFER_SIZE )
+					m_nReadFileBufferSize = xml->content(n2).asULL();
 			}
 		}
 		/* parse server entrys */
