@@ -555,6 +555,28 @@ public:
 	/** Set remote encoding setting */
 	void SetRemoteEncoding( CString name );
 	
+	/** Get UserIP2 support enabled */
+	bool GetUserIP2Enabled() const;
+	/** Set UserIP2 support enabled */
+	void SetUserIP2Enabled( bool b );
+	
+	/** Get ZLine support enabled */
+	bool GetZLineEnabled() const;
+	/** Set ZLine support enabled */
+	void SetZLineEnabled( bool b );
+	
+	/** Get session starting value of total downloaded */
+	ulonglong GetStartDownloaded() const;
+	/** Get session starting value of total uploaded */
+	ulonglong GetStartUploaded() const;
+	/** Reset session totals to zero */
+	void ResetSessionStats();
+	
+	/** Get read file buffer size */
+	ulonglong GetReadFileBufferSize() const;
+	/** Set read file buffer size */
+	void SetReadFileBufferSize( ulonglong newsize );
+	
 	/** */
 	CString AliasToPath( CString file );
 
@@ -699,6 +721,16 @@ private:
 	bool m_bDontShareDotFiles;
 	/** The remote text encoding */
 	CString m_sRemoteEncoding;
+	/** If true, enable UserIP2 support (get IP address from hub) */
+	bool m_bUserIP2Enabled;
+	/** If true, enable ZLine support */
+	bool m_bEnableZLine;
+	/** Session starting value of total downloaded */
+	ulonglong m_nStartDownloaded;
+	/** Session starting value of total uploaded */
+	ulonglong m_nStartUploaded;
+	/** Size of each file read when uploading */
+	ulonglong m_nReadFileBufferSize;
 
 protected:
 	/** */
@@ -1037,5 +1069,32 @@ inline CString CConfig::GetRemoteEncoding() const
 /** */
 inline void CConfig::SetRemoteEncoding( CString name )
 { Thread.Lock(); m_sRemoteEncoding = name; Thread.UnLock(); }
+/** */
+inline bool CConfig::GetUserIP2Enabled() const
+{ return m_bUserIP2Enabled; }
+/** */
+inline void CConfig::SetUserIP2Enabled( bool b )
+{ Thread.Lock(); m_bUserIP2Enabled = b; Thread.UnLock(); }
+/** */
+inline bool CConfig::GetZLineEnabled() const
+{ return m_bEnableZLine; }
+/** */
+inline void CConfig::SetZLineEnabled( bool b )
+{ Thread.Lock(); m_bEnableZLine = b; Thread.UnLock(); }
+/** */
+inline ulonglong CConfig::GetStartDownloaded() const
+{ return m_nStartDownloaded; }
+/** */
+inline ulonglong CConfig::GetStartUploaded() const
+{ return m_nStartUploaded; }
+/** */
+inline void CConfig::ResetSessionStats()
+{ Thread.Lock(); m_nStartDownloaded = 0; m_nStartUploaded = 0; Thread.UnLock(); }
+/** */
+inline ulonglong CConfig::GetReadFileBufferSize() const
+{ return m_nReadFileBufferSize; }
+/** */
+inline void CConfig::SetReadFileBufferSize( ulonglong newsize )
+{ Thread.Lock(); m_nReadFileBufferSize = newsize; Thread.UnLock(); }
 
 #endif
