@@ -179,6 +179,7 @@ struct filetype FileType[] = {
 	{ "ASF",   eftVIDEO },
 	{ "ASX",   eftVIDEO },
 	{ "DAT",   eftVIDEO },
+	{ "FLV",   eftVIDEO },
 	{ "MKV",   eftVIDEO },
 	{ "MOV",   eftVIDEO },
 	{ "MOVIE", eftVIDEO },
@@ -640,7 +641,10 @@ void CFileManager::ThreadCreateShareList()
 		printf("ready create sharelist\n");
 
 		m_pShareList->SetIndexBuffer( m_sShareIndexBuffer );
-		m_pShareList->CreateList(m_pSearchIndex);
+		
+		// m_pShareList->CreateList will be called after hashing finishes
+		// don't call it here to avoid serving up filelists without TTHs in
+		//m_pShareList->CreateList(m_pSearchIndex);
 
 		if ( m_pFileNameList )
 		{
