@@ -68,7 +68,7 @@
 #define DC_S_MESSAGE_CAPABILITIES	"$Capabilities "
 #define DC_S_MESSAGE_HUB_TOPIC		"$HubTopic "
 #define DC_S_MESSAGE_GET_NET_INFO	"$GetNetInfo"
-#define DC_S_MESSGAE_USER_IP		"$UserIP "
+#define DC_S_MESSAGE_USER_IP		"$UserIP "
 // ignored messages
 #define DC_S_MESSAGE_MULTISEARCH	"$MultiSearch "
 #define DC_S_MESSAGE_USER_COMMAND	"$UserCommand "
@@ -80,6 +80,7 @@
 #define DC_S_USER_COMMAND		"$UserCommand " // ??? ( $UserCommand 0 / $UserCommand 1 Read Help$<%[mynick]> +help&#124;<%[mynick]> -help&#124; )
 
 #define DC_S_MESSAGE_UGETBLOCK		"$UGetBlock "
+#define DC_S_MESSAGE_UGETZBLOCK		"$UGetZBlock "
 // ZLIB compression messages
 #define DC_S_MESSAGE_GETZBLOCK		"$GetZBlock "
 #define DC_S_MESSAGE_SENDING		"$Sending "
@@ -89,6 +90,9 @@
 #define DC_S_MESSAGE_ADCGET		"$ADCGET "
 #define DC_S_MESSAGE_ADCSND		"$ADCSND "
 
+// ZLine message - just $ZOn|
+#define DC_S_MESSAGE_ZON		"$ZOn"
+
 CMessageHandler::CMessageHandler()
 {
 	CString from = "ISO-8859-1";
@@ -285,6 +289,12 @@ eDCMessage CMessageHandler::Parse( const CString * sMessage, int & pointer, CObj
 
 			return DC_MESSAGE_GET;
 		}
+		else if ( GetContent(DC_S_MESSAGE_UGETZBLOCK,&t,sContent) != -1 )
+		{
+			Object = ParseUGetZBlock(sContent);
+			
+			return DC_MESSAGE_GET;
+		}
 		else if ( GetContent(DC_S_MESSAGE_ERROR,&t,sContent) != -1 )
 		{
 			Object = ParseError(sContent);
@@ -409,7 +419,7 @@ eDCMessage CMessageHandler::Parse( const CString * sMessage, int & pointer, CObj
 		{
 			Object = ParseGetZBlock(sContent);
 
-			return DC_MESSAGE_GETZBLOCK;
+			return DC_MESSAGE_GET;
 		}
 		else if ( GetContent(DC_S_MESSAGE_FAILED,&t,sContent) != -1 )
 		{
@@ -429,6 +439,18 @@ eDCMessage CMessageHandler::Parse( const CString * sMessage, int & pointer, CObj
 			
 			return DC_MESSAGE_ADCSND;
 		}
+		else if ( GetContent(DC_S_MESSAGE_USER_IP,&t,sContent) != -1 )
+		{
+			Object = ParseUserIP(sContent);
+			
+			return DC_MESSAGE_USERIP;
+		}
+		else if (  GetContent(DC_S_MESSAGE_ZON,&t,sContent) != -1 )
+		{
+			Object = new CMessageZLine();
+			
+			return DC_MESSAGE_ZLINE;
+		}
 		else
 		{
 			printf("!!!!!!!!!!!!!! unknown start ");
@@ -1499,33 +1521,60 @@ CObject * CMessageHandler::ParseFileLength( CString sContent )
 /** */
 CObject * CMessageHandler::ParseGet( CString sContent )
 {
-	int i;
-	CString s,s1;
-	CMessageGet * msg=0;
-
-	if ( (i=sContent.Find('$')) < 0 )
-		return 0;
-
-	if ( (msg = new CMessageGet()) != 0 )
+	int last,secondlast;
+	CString pos, size;
+	CMessageGet * msg = 0;
+	
+	// last $ will usually be 1-starting based file position
+	// but will be size if using valknut CHUNK extension
+	last = sContent.FindRev('$');
+	
+	if ( last < 0 )
 	{
-		msg->m_sFilename = m_pIconv->encode(sContent.Mid(0,i));
-
-		s = sContent.Mid(i+1,sContent.Length()-i-1);
-
-		// check if special get message "POS$SIZE"
-		if ( (i=s.Find('$')) < 0 )
+		return 0;
+	}
+	
+	msg = new CMessageGet();
+	
+	if ( msg == 0 )
+	{
+		return 0;
+	}
+	
+	// second last $ may be in the filename
+	// but will be file position if using valknut CHUNK extension
+	secondlast = sContent.FindRev('$',last-1);
+	
+	if ( secondlast < 0 )
+	{
+		pos = sContent.Mid(last+1);
+		size = "";
+		secondlast = last;
+	}
+	else
+	{
+		size = sContent.Mid(last+1);
+		pos = sContent.Mid(secondlast+1,last-(secondlast+1));
+		
+		if ( pos.asULL() == 0 ) // $Get starts at 1
 		{
-			msg->m_nPos = s.asULL();
-		}
-		else
-		{
-			s1 = s.Mid(0,i);
-			msg->m_nPos = s1.asULL();
-			s1 = s.Mid(i+1,s.Length()-i-1);
-			msg->m_nSize = s1.asULL();
+			pos = size;
+			size = "";
+			secondlast = last;
 		}
 	}
-
+	
+	msg->m_sFilename = m_pIconv->encode(sContent.Mid(0,secondlast));
+	
+	msg->m_nPos = pos.asULL();
+	
+	if ( size != "" )
+	{
+		msg->m_nSize = size.asULL();
+	}
+	
+	//printf( "name=%s pos=%llu size=%llu\n", msg->m_sFilename.Data(), msg->m_nPos, msg->m_nSize );
+	
 	return msg;
 }
 
@@ -1560,6 +1609,19 @@ CObject * CMessageHandler::ParseUGetBlock( CString sContent )
 	return msg;
 }
 
+/** */
+CObject * CMessageHandler::ParseUGetZBlock( CString sContent )
+{
+	CMessageGet * msg = (CMessageGet*) ParseUGetBlock(sContent);
+	
+	if ( msg != 0 )
+	{
+		msg->m_bZLib = true;
+	}
+	
+	return msg;
+}
+
 /** */
 CObject * CMessageHandler::ParseError( CString sContent )
 {
@@ -1650,6 +1712,7 @@ CObject * CMessageHandler::ParseLogedIn( CString sContent )
 #define DC_SUPPORT_TAG_CLIENTID		"ClientID "
 #define DC_SUPPORT_TAG_SECUREDEXECUTOR	"SecuredExecutor "
 #define DC_SUPPORT_TAG_FEED		"Feed "
+#define DC_SUPPORT_TAG_ZLINE		"ZLine "
 
 /** */
 CObject * CMessageHandler::ParseSupports( CString sContent )
@@ -1658,6 +1721,11 @@ CObject * CMessageHandler::ParseSupports( CString sContent )
 	int i,i1;
 	CString s;
 
+	if ( sContent.Right(1) != " " )
+	{
+		sContent += " ";
+	}
+
 	if ( (msg = new CMessageSupports()) != 0 )
 	{
 		msg->m_sContent = sContent;
@@ -1770,6 +1838,10 @@ CObject * CMessageHandler::ParseSupports( CString sContent )
 			{
 				msg->m_bFeed = TRUE;
 			}
+			else if ( s == DC_SUPPORT_TAG_ZLINE )
+			{
+				msg->m_bZLine = TRUE;
+			}
 			else
 			{
 				printf("Wrong support tag: '%s'\n",s.Data());
@@ -1848,7 +1920,16 @@ CObject * CMessageHandler::ParseYourIP( CString sContent )
 /** */
 CObject * CMessageHandler::ParseGetZBlock( CString sContent )
 {
-	return 0;
+	CMessageGet * msg = (CMessageGet*) ParseUGetBlock(sContent);
+	
+	if ( msg != 0 )
+	{
+		msg->m_bUGet = false;
+		msg->m_bZLib = true;
+		msg->m_sFilename = m_pIconv->encode(msg->m_sFilename);
+	}
+	
+	return msg;
 }
 
 /** */
@@ -1952,3 +2033,29 @@ CObject * CMessageHandler::ParseADCSnd( CString sContent )
 	
 	return msg;
 }
+
+/** $UserIP nick ip| */
+CObject * CMessageHandler::ParseUserIP( CString sContent )
+{
+	CMessageUserIP * msg = 0;
+	int i;
+	
+	i = sContent.Find(' ');
+	if ( i < 0 )
+	{
+		return 0;
+	}
+	
+	msg = new CMessageUserIP();
+	
+	if ( msg != 0 )
+	{
+		msg->m_sNick = m_pIconv->encode(sContent.Mid(0,i));
+		
+		msg->m_sIP = sContent.Mid(i+1,sContent.Length()-i-1);
+		
+		//printf("ParseUserIP: nick=%s IP=%s\n", msg->m_sNick.Data(), msg->m_sIP.Data() );
+	}
+	
+	return msg;
+}
