@@ -78,20 +78,19 @@ bool CLogFile::Write( CString file, eLogType type, char * message, ... )
 bool CLogFile::Write( CString file, eLogType /*type*/, CString message )
 {
 	bool res = FALSE;
-	CString timestamp;
+	char timestamp[23];
 	struct tm * t;
 	time_t ti;
 	FILE * logfile;
-
+	CString newline;
+	
 	LogfileThread.Lock();
 
 	ti = time(0);
 	t  = localtime(&ti);
-
-	timestamp = "[" + CString().setNum(t->tm_hour) + ":" +
-			CString().setNum(t->tm_min) + ":" +
-			CString().setNum(t->tm_sec) + "] ";
-
+	
+	strftime(timestamp, sizeof(timestamp), "[%F.%H:%M:%S] ", t);
+	
 	// TODO: fix logfiles and add option to set logpath ...
 /*
 	if ( CConfig::Instance() )
@@ -101,10 +100,10 @@ bool CLogFile::Write( CString file, eLogType /*type*/, CString message )
 */
 	if ( (logfile = fopen(file.Data(),"a")) != 0 )
 	{
-		fwrite(timestamp.Data(),timestamp.Length(),1,logfile);
+		fwrite(timestamp,22,1,logfile);
 		fwrite(message.Data(),message.Length(),1,logfile);
-		timestamp = "\n";
-		fwrite(timestamp.Data(),timestamp.Length(),1,logfile);
+		newline = "\n";
+		fwrite(newline.Data(),newline.Length(),1,logfile);
 		fclose(logfile);
 		res = TRUE;
 	}
