@@ -1421,9 +1421,11 @@ int CSocket::Read( char * buffer, int len, int sec_timeout, int usec_timeout )
 			s = "RECV:" + CString().setNum(l);
 			if( l <= len )
 			{
+				char prev = buffer[l];
 				s+="\n";
 				buffer[l]=0;
 				s += buffer;
+				buffer[l] = prev;
 			}
 			CLogFile::Write("dcsocket.log",eltINFO,s);
 		}
@@ -1572,9 +1574,11 @@ int CSocket::Write( const unsigned char * buffer, int len, int sec_timeout, int
 		if ( i > 0 )
 		{
 			CString s;
+			char prev = ((char*)buffer)[i];
 			s = "SEND:" + CString().setNum(len)+"\n";
 			((char*)buffer)[i]=0;
 			s += (char*)buffer;
+			((char*)buffer)[i] = prev;
 			CLogFile::Write("dcsocket.log",eltINFO,s);
 		}
 	}
