@@ -51,7 +51,7 @@ SSL_CTX * CSSL::InitClientCTX()
 	SSL_METHOD *method;
 	SSL_CTX *ctx = NULL;
 
-	OpenSSL_add_all_algorithms();		/* Load cryptos, et.al. */
+	SSL_library_init();			/* Load cryptos, et.al. */
 	SSL_load_error_strings();		/* Bring in and register error messages */
 	
 	method = SSLv23_client_method();		/* Create new client-method instance */
@@ -77,7 +77,7 @@ SSL_CTX * CSSL::InitServerCTX()
 	SSL_METHOD *method;
 	SSL_CTX *ctx = NULL;
 
-	OpenSSL_add_all_algorithms();		/* Load cryptos, et.al. */
+	SSL_library_init();			/* Load cryptos, et.al. */
 	SSL_load_error_strings();		/* Bring in and register error messages */
 	
 	method = SSLv23_server_method();		/* Create new client-method instance */
