@@ -31,6 +31,7 @@
 #include <dclib/cconfig.h>
 #include <dclib/core/cmanager.h>
 #include <dclib/csearchindex.h>
+#include <dclib/core/ciconv.h>
 
 #include "cquerymanager.h"
 
@@ -146,6 +147,7 @@ bool CQueryManager::AddResult( CQuerySendObject * querysendobject, CQueryObject
 {
 	CString s, result;
 	int slot_max,slot_free;
+	CIconv ciconv( "UTF-8", CConfig::Instance()->GetRemoteEncoding() );
 
 	// get slot info
 	slot_max  = CConfig::Instance()->GetMaxUpload();
@@ -168,10 +170,10 @@ bool CQueryManager::AddResult( CQuerySendObject * querysendobject, CQueryObject
 	}
 
 	// create the searchresult
-	s += filename;
+	s += ciconv.encode(filename);
 	s  = s.Replace('/',"\\");
 	result  = "$SR ";
-	result += queryobject->sNick + " ";
+	result += ciconv.encode(queryobject->sNick) + " ";
 	result += s + "\x5";
 	result += CString().setNum(fbo->m_nSize) + " ";
 	result += CString().setNum(slot_free) + "/" + CString().setNum(slot_max);
@@ -181,7 +183,7 @@ bool CQueryManager::AddResult( CQuerySendObject * querysendobject, CQueryObject
 	if ( hash != "" )
 		result += hash;
 	else
-		result += queryobject->sHubName;
+		result += ciconv.encode(queryobject->sHubName);
 	result += " ";
 
 	result += "(" + queryobject->sHubHost + ")";
@@ -189,12 +191,13 @@ bool CQueryManager::AddResult( CQuerySendObject * querysendobject, CQueryObject
 	// result found ! send it ...
 	if ( queryobject->pMessageSearch->m_bLocal == TRUE )
 	{
-		result += "\x5" + queryobject->pMessageSearch->m_sSource + "|";
+		result += "\x5" + ciconv.encode(queryobject->pMessageSearch->m_sSource) + "|";
 		querysendobject->m_pList->Add("r", new CString(result));
 //		printf("ADD L RESULT: '%s'\n",result.Data());
 	}
 	else
 	{
+		result += "|";
 		querysendobject->m_pList->Add("r", new CString(result));
 //		printf("ADD G RESULT: '%s'\n",result.Data());
 	}
@@ -252,7 +255,7 @@ void CQueryManager::SendResults()
 
 			while ( querysendobject->m_pList->Next( (CObject*&) s) == 1 )
 			{
-				if ( CConnectionManager::Instance()->SendStringToConnectedServers(*s,querysendobject->m_sSource) == 0 )
+				if ( CConnectionManager::Instance()->SendStringToConnectedServers(*s,querysendobject->m_sSource,FALSE) == 0 )
 				{
 					m_nResultCountError++;
 					
