@@ -39,9 +39,12 @@
 #include <dclib/core/cbz.h>
 #include <dclib/core/che3.h>
 #include <dclib/cfilemanager.h>
-#include <dclib/hash/tigertree.h>
 #include <dclib/core/cbase32.h>
 
+#include <dclib/hash/Encoder.h>
+#include <dclib/hash/TigerHash.h>
+#include <dclib/hash/MerkleTree.h>
+
 #include "csearchindex.h"
 
 /** */
@@ -58,6 +61,8 @@ CSearchIndex::CSearchIndex()
 	m_pHashFileBaseArray = new CByteArray();
 	m_pHashPathBaseArray = new CByteArray();
 	m_pHashIndex = new CByteArray();
+	//m_pHashLeaves = new CByteArray();
+	m_pHashLeaves = 0;
 	
 	memset(m_pSearchArray,0,sizeof(m_pSearchArray));
 
@@ -80,6 +85,11 @@ CSearchIndex::~CSearchIndex()
 	delete m_pHashFileBaseArray;
 	delete m_pHashPathBaseArray;
 	delete m_pHashIndex;
+	if (m_pHashLeaves != 0)
+	{
+		DPRINTF("CSearchIndex: warning hashleaves not closed!\n");
+		CloseHashLeaves();
+	}
 }
 
 void CSearchIndex::Reset()
@@ -108,6 +118,9 @@ void CSearchIndex::ResetHashIndex()
 	m_pHashFileBaseArray->SetSize(0);
 	m_pHashPathBaseArray->SetSize(0);
 	m_pHashIndex->SetSize(0);
+	CloseHashLeaves();
+	m_pHashLeaves = new CByteArray();
+	CloseHashLeaves();
 }
 
 /** */
@@ -164,7 +177,7 @@ CStringList * CSearchIndex::SearchHash( unsigned char * hash )
 				sl->Add( *ps, ps );
 			}
 		}
-		hi++;
+		hi+=TigerTree::HASH_SIZE;
 	}
 
 	return sl;
@@ -249,7 +262,18 @@ bool CSearchIndex::LoadIndex()
 			err = TRUE;
 		}
 	}
-
+	if ( err == FALSE )
+	{
+		if ( OpenHashLeaves() == FALSE )
+		{
+			err = TRUE;
+		}
+		else
+		{
+			CloseHashLeaves();
+		}
+	}
+	
 	if ( err == TRUE )
 	{
 		ResetHashIndex();
@@ -352,6 +376,11 @@ void CSearchIndex::SaveIndex()
 	m_pHashFileBaseArray->SaveToFile(CConfig::Instance()->GetConfigPath()+"hashfilebase.bin");
 	m_pHashPathBaseArray->SaveToFile(CConfig::Instance()->GetConfigPath()+"hashpathbase.bin");
 	m_pHashIndex->SaveToFile(CConfig::Instance()->GetConfigPath()+"hashindex.bin");
+	
+	if (m_pHashLeaves != 0)
+	{
+		CloseHashLeaves();
+	}
 }
 
 /** */
@@ -403,8 +432,8 @@ bool CSearchIndex::FindHash( unsigned char * hash, ulonglong * hi )
 {
 	ulonglong i;
 	
-	for(i=*hi;i<m_pHashIndex->Size();i+=TIGERSIZE)
-		if ( memcmp(hash,m_pHashIndex->Data()+i,TIGERSIZE) == 0 )
+	for(i=*hi;i<m_pHashIndex->Size();i+=TigerTree::HASH_SIZE)
+		if ( memcmp(hash,m_pHashIndex->Data()+i,TigerTree::HASH_SIZE) == 0 )
 		{
 			*hi = i;
 			return TRUE;
@@ -419,10 +448,12 @@ CString CSearchIndex::GetHash( ulonglong hbi )
 	CString s;
 	CBase32 base32;
 	CByteArray dst,src;
-	
-	if ( hbi < m_pHashIndex->Size() )
+	struct hashbaseobject * hbo;
+
+	if ( hbi < m_pHashBaseArray->Size() )
 	{
-		src.Append(m_pHashIndex->Data()+hbi,TIGERSIZE);
+		hbo = (struct hashbaseobject *)(m_pHashBaseArray->Data()+hbi);
+		src.Append(m_pHashIndex->Data()+hbo->m_nHashIndex,TigerTree::HASH_SIZE);
 		base32.Encode( &dst, &src );
 		if ( dst.Size() > 0 )
 		{
@@ -519,12 +550,12 @@ bool CSearchIndex::Compare( struct filebaseobject * fbo, struct hashbaseobject *
 }
 
 /** */
-void CSearchIndex::AddHashIndex( ulonglong filebaseindex, unsigned char * hash )
+void CSearchIndex::AddHashIndex( ulonglong filebaseindex, unsigned char * hash, unsigned char * leaves, unsigned int lsize )
 {
 	struct filebaseobject fbo;
 	struct hashbaseobject hbo;
 	char * c;
-	ulonglong hi,hbi;
+	ulonglong i,hi,hbi;
 	
 	if ( !GetFileBaseObject(filebaseindex,&fbo) )
 	{
@@ -551,7 +582,19 @@ void CSearchIndex::AddHashIndex( ulonglong filebaseindex, unsigned char * hash )
 	hbo.m_nFileIndex = m_pHashFileBaseArray->Size();
 	hbo.m_nPathIndex = m_pHashPathBaseArray->Size();
 	hbo.m_nHashIndex = m_pHashIndex->Size();
-
+	
+	if ( (lsize == 0) || (leaves == 0) )
+	{
+		hbo.m_nHashLeavesIndex = (ulonglong)-1;
+	}
+	else
+	{
+		i = lsize;
+		hbo.m_nHashLeavesIndex = m_pHashLeaves->Size();
+		m_pHashLeaves->Append((unsigned char*)&i,sizeof(i));
+		m_pHashLeaves->Append(leaves,lsize);
+	}
+	
 	fbo.m_nHashIndex = m_pHashBaseArray->Size();
 
 	m_pHashBaseArray->Append((const char*)&hbo, sizeof(struct hashbaseobject));
@@ -559,7 +602,7 @@ void CSearchIndex::AddHashIndex( ulonglong filebaseindex, unsigned char * hash )
 	m_pHashFileBaseArray->Append(c,strlen(c)+1);
 	c = (char*)m_pPathBaseArray->Data()+fbo.m_nPathIndex;
 	m_pHashPathBaseArray->Append(c,strlen(c)+1);
-	m_pHashIndex->Append( hash, TIGERSIZE );
+	m_pHashIndex->Append( hash, TigerTree::HASH_SIZE );
 
 	UpdateIndex(filebaseindex,&fbo);
 }
@@ -715,3 +758,249 @@ void CSearchIndex::AddIndex( CString & s, ulonglong filebaseindex )
 
 	m_pSearchIndex->Append( (const char*)&fio, sizeof(struct fileindexobject) );
 }
+
+/** */
+CByteArray * CSearchIndex::GetHashLeaves( CString tth )
+{
+	CByteArray * ba = 0;
+	ulonglong hi = 0, hbi = 0, hli = 0, lsize = 0;
+	struct hashbaseobject * hbo = 0;
+	CBase32 base32;
+	CByteArray dst;
+	
+	if (base32.Decode(&dst, &tth) != TigerTree::HASH_SIZE)
+	{
+		DPRINTF("GetHashLeaves: Decoded TTH length != TigerTree::HASH_SIZE\n");
+		return 0;
+	}
+	
+	if ( FindHash(dst.Data(), &hi) )
+	{
+		//DPRINTF("GetHashLeaves: HashIndex=%llu\n", hi);
+		
+		if ( HashBaseIndexFromHashIndex( hi, &hbi ) )
+		{
+			//DPRINTF("GetHashLeaves: HashBaseIndex=%llu\n", hbi);
+			hbo = (struct hashbaseobject *)(m_pHashBaseArray->Data()+hbi);
+			
+			// hash leaves index
+			hli = hbo->m_nHashLeavesIndex;
+			
+			if (hli == (ulonglong)-1)
+			{
+				DPRINTF("GetHashLeaves: No Leaves available.\n");
+				return 0;
+			}
+			
+			//DPRINTF("GetHashLeaves: HashLeavesIndex=%llu\n", hli);
+			
+			/* Old code for in memory hash leaves*/
+			/* lsize = * (ulonglong *) (m_pHashLeaves->Data()+hli);
+			
+			//DPRINTF("GetHashLeaves: lsize=%llu\n", lsize);
+			
+			ba = new CByteArray();
+			ba->Append( (m_pHashLeaves->Data()+(hli + sizeof(lsize))), lsize );
+			
+			return ba; */
+			
+			/* New code for on disk hash leaves */
+			CFile leaffile;
+			
+			leaffile.Open(CConfig::Instance()->GetConfigPath()+"hashleaves.bin", IO_READONLY);
+			leaffile.Seek(hli, 0);
+			
+			long read1 = 0, read2 = 0;
+			read1 = leaffile.Read( (char *)&lsize, sizeof(lsize) );
+			if (read1 != sizeof(lsize))
+			{
+				DPRINTF("GetHashLeaves: warning didn't read lsize!\n");
+			}
+			
+			//DPRINTF("GetHashLeaves: lsize=%llu\n", lsize);
+			
+			leaffile.Seek(sizeof(lsize), hli);
+			
+			ba = new CByteArray();
+			
+			char * buf = new char[lsize];
+			read2 = leaffile.Read( buf, lsize );
+			if (read2 != lsize)
+			{
+				DPRINTF("GetHashLeaves: warning didn't read enough data!\n");
+			}
+			
+			leaffile.Close();
+			
+			ba->Append( buf, lsize );
+			
+			delete buf;
+			
+			return ba;
+		}
+		else
+		{
+			return 0;
+		}
+	}
+	else
+	{
+		return 0;
+	}
+}
+
+/** */
+bool CSearchIndex::OpenHashLeaves()
+{
+	bool res;
+	
+	if (m_pHashLeaves != 0)
+	{
+		DPRINTF("CSearchIndex::OpenHashLeaves: leaves already open!\n");
+		CloseHashLeaves();
+	}
+	
+	m_pHashLeaves = new CByteArray();
+	res = m_pHashLeaves->LoadFromFile(CConfig::Instance()->GetConfigPath()+"hashleaves.bin");
+	
+	return res;
+}
+
+/** */
+bool CSearchIndex::CloseHashLeaves()
+{
+	bool res;
+	
+	if (m_pHashLeaves != 0)
+	{
+		m_pHashLeaves->SaveToFile(CConfig::Instance()->GetConfigPath()+"hashleaves.bin");
+		delete m_pHashLeaves;
+		m_pHashLeaves = 0;
+		res = TRUE;
+	}
+	else
+	{
+		DPRINTF("CSearchIndex::CloseHashLeaves: leaves not open!\n");
+		res = FALSE;
+	}
+	
+	return res;
+}
+
+/** */
+void CSearchIndex::RebuildLists()
+{
+	CByteArray * pNewBaseArray     = new CByteArray();
+	CByteArray * pNewFileBaseArray = new CByteArray();
+	CByteArray * pNewPathBaseArray = new CByteArray();
+	
+	CByteArray * pNewHashBaseArray     = new CByteArray();
+	CByteArray * pNewHashFileBaseArray = new CByteArray();
+	CByteArray * pNewHashPathBaseArray = new CByteArray();
+	CByteArray * pNewHashIndex         = new CByteArray();
+	CByteArray * pNewHashLeaves        = new CByteArray(); 
+	
+	ulonglong count = IndexCount();
+	
+	CFile file;
+	CBase32 b32;
+	
+	for (ulonglong i = 0; i < count; i++)
+	{
+		struct filebaseobject fbo;
+		
+		GetFileBaseObject( i, &fbo );
+		
+		CString vpath = (char*) m_pPathBaseArray->Data()+fbo.m_nPathIndex;
+		
+		CString dirsep = DIRSEPARATOR;
+		
+		CString filename = (char*) m_pFileBaseArray->Data()+fbo.m_nFileIndex;
+		
+		CString realpath = CConfig::Instance()->AliasToPath(vpath+dirsep+filename);
+		
+		// check if file no longer shared (not actually needed)
+		if ( realpath == "" )
+		{
+			DPRINTF( "%s%s%s no longer shared, entry removed\n", vpath.Data(), dirsep.Data(), filename.Data() );
+			continue;
+		}
+		
+		DPRINTF( "%s ", realpath.Data() );
+		
+		// if file exists
+		if ( file.Open(realpath, IO_READONLY) == TRUE )
+		{
+			file.Close();
+			DPRINTF("exists ");
+			
+			// get TTH and hash leaves
+			struct hashbaseobject * oldhbo = (struct hashbaseobject *) (m_pHashBaseArray->Data()+fbo.m_nHashIndex);
+			CByteArray dst,src;
+			src.Append( m_pHashIndex->Data()+oldhbo->m_nHashIndex, TigerTree::HASH_SIZE );
+			b32.Encode( &dst, &src );
+			CString tth;
+			tth.Set( (const char*)dst.Data(), dst.Size() );
+			
+			DPRINTF("tth=%s\n", tth.Data() );
+			
+			CByteArray * hashleaves = GetHashLeaves( tth );
+			ulonglong lsize = hashleaves->Size();
+			
+			fbo.m_nFileIndex = pNewFileBaseArray->Size();
+			fbo.m_nPathIndex = pNewPathBaseArray->Size();
+			fbo.m_nHashIndex = pNewHashBaseArray->Size();
+			
+			pNewBaseArray->Append( (const char*)&fbo, sizeof(struct filebaseobject) );
+			pNewFileBaseArray->Append( filename.Data(), filename.Length() + 1 );
+			pNewPathBaseArray->Append( vpath.Data(), vpath.Length() + 1 );
+			
+			struct hashbaseobject hbo;
+			hbo.m_nSize            = fbo.m_nSize;
+			hbo.m_nPathIndex       = pNewHashPathBaseArray->Size();
+			hbo.m_nFileIndex       = pNewHashFileBaseArray->Size();
+			hbo.m_nHashIndex       = pNewHashIndex->Size();
+			hbo.m_nHashLeavesIndex = pNewHashLeaves->Size();
+			hbo.m_tModTime         = fbo.m_tModTime;
+			
+			pNewHashBaseArray->Append( (const char*)&hbo, sizeof(struct hashbaseobject) );
+			pNewHashPathBaseArray->Append( vpath.Data(), vpath.Length() + 1 );
+			pNewHashFileBaseArray->Append( filename.Data(), filename.Length() + 1 );
+			pNewHashIndex->Append( src.Data(), TigerTree::HASH_SIZE );
+			pNewHashLeaves->Append( (unsigned char*)&lsize, sizeof(lsize) );
+			pNewHashLeaves->Append( hashleaves->Data(), lsize );
+		}
+		else
+		{
+			DPRINTF("does not exist, entry removed\n");
+		}
+	}
+	
+	// delete old lists
+	delete m_pBaseArray;
+	delete m_pFileBaseArray;
+	delete m_pPathBaseArray;
+	
+	delete m_pHashBaseArray;
+	delete m_pHashFileBaseArray;
+	delete m_pHashPathBaseArray;
+	delete m_pHashIndex;
+	if ( m_pHashLeaves != 0 )
+	{
+		delete m_pHashLeaves;
+	}
+	
+	// replace old lists with new lists
+	m_pBaseArray     = pNewBaseArray;
+	m_pFileBaseArray = pNewFileBaseArray;
+	m_pPathBaseArray = pNewPathBaseArray;
+	
+	m_pHashBaseArray     = pNewHashBaseArray;
+	m_pHashFileBaseArray = pNewHashFileBaseArray;
+	m_pHashPathBaseArray = pNewHashPathBaseArray;
+	m_pHashIndex         = pNewHashIndex;
+	m_pHashLeaves        = pNewHashLeaves;
+	
+	// SaveIndex is called by CFileManager::ThreadRebuildLists
+	// which will close the hash leaves array
+}
