@@ -43,7 +43,10 @@
 
 #define SEARCH_TIMEOUT				60
 #define SEARCH_CLIENT_TIMEOUT			60
-#define SEARCH_SPAM_TIMEOUT			60
+
+// no longer defined because we are using a configured value
+//define SEARCH_SPAM_TIMEOUT			60
+
 #define SEARCH_PASSIVE_CLIENT_REMOVE_TIMEOUT	5
 #define SEARCH_ACTIVE_CLIENT_REMOVE_TIMEOUT	60
 
@@ -283,7 +286,7 @@ int CSearchManager::CallBackManager( CObject *, CObject * )
 		{
 			case esmCONNECTEDSINGLE:
 			case esmCONNECTEDALL:
-				if ( (time(0) - m_tTimeoutTime) >= SEARCH_SPAM_TIMEOUT )
+				if ( (time(0) - m_tTimeoutTime) >= (CConfig::Instance()->GetAutoSearchInterval()) )
 				{
 					if ( DoSearch(0) == FALSE )
 					{
@@ -366,6 +369,26 @@ bool CSearchManager::AddClient()
 	
 	DPRINTF("add client: '%s'\n",m_sCurrentHub->Data());
 
+	if (m_sCurrentHub != 0)
+	{
+		char timestamp[23];
+		struct tm * t;
+		time_t ti;
+		
+		ti = time(0);
+		t  = localtime(&ti);
+		strftime(timestamp, sizeof(timestamp), "[%F.%H:%M:%S] ", t);
+		
+		CMessageLog * msglog = new CMessageLog();
+		msglog->sMessage = CString(timestamp) + "Add client on " + *m_sCurrentHub;
+		//DPRINTF("%s\n", msglog->sMessage.Data());
+		SendObject(msglog);
+	}
+	//else
+	//{
+	//	DPRINTF("m_sCurrentHub == 0\n");
+	//}
+
 	client = new CSearchClient();
 	
 	// set default values for search
@@ -428,7 +451,7 @@ void CSearchManager::UpdateClients()
 			// send search
 			else if ( client->m_bSearchEnable == TRUE )
 			{
-				if ( (time(0) - client->m_tSearchTimeout) >= SEARCH_SPAM_TIMEOUT )
+				if ( (time(0) - client->m_tSearchTimeout) >= (CConfig::Instance()->GetAutoSearchInterval()) )
 				{
 					if ( DoSearch(client) == FALSE )
 					{
