@@ -52,8 +52,6 @@
 
 #include "ctransfer.h"
 
-#define READ_FILE_BUFFER_SIZE	(40*1024)
-
 CTransfer::CTransfer( bool listener )
 {
 	pCallback = 0;
@@ -97,6 +95,15 @@ CTransfer::CTransfer( bool listener )
 	memset(&m_AverageTransferRate,0,sizeof(m_AverageTransferRate));
 	m_AverageTransferRate.index=0;
 	gettimeofday(&m_AverageTransferRate.tv[0],NULL);
+	
+	if ( CConfig::Instance() )
+	{
+		m_nReadFileBufferSize = CConfig::Instance()->GetReadFileBufferSize();
+	}
+	else
+	{
+		m_nReadFileBufferSize = 40960;
+	}
 }
 
 CTransfer::~CTransfer()
@@ -471,7 +478,7 @@ int CTransfer::StartDownload( CString dstfile, ulonglong startposition, ulonglon
 }
 
 /** */
-int CTransfer::StartUpload( CString dstfile, ulonglong length, ulonglong pos, ulonglong chunksize, CString srcfile, bool uget, bool adcget, CString adcTTH )
+int CTransfer::StartUpload( CString dstfile, ulonglong length, ulonglong pos, ulonglong chunksize, CString srcfile, bool uget, bool adcget, CString adcTTH, bool compress )
 {
 	if ( (srcfile == "" ) && (eMedium != eltBUFFER) )
 	{
@@ -509,6 +516,15 @@ int CTransfer::StartUpload( CString dstfile, ulonglong length, ulonglong pos, ul
 	/** set starttime */
 	InitTime();
 	
+	if ( compress )
+	{
+		m_nDataType = 2;
+	}
+	else
+	{
+		m_nDataType = 0;
+	}
+	
 	if (adcget)
 	{
 		if (GetMedium() == eltTTHL)
@@ -520,21 +536,24 @@ int CTransfer::StartUpload( CString dstfile, ulonglong length, ulonglong pos, ul
 			}
 			else
 			{
-				SendADCSnd( "tthl", "TTH/" + adcTTH, 0, treesize);
+				SendADCSnd( "tthl", "TTH/" + adcTTH, 0, treesize, compress );
 			}
 		}
 		else if (adcTTH == DC_USER_FILELIST_XMLBZ)
 		{
 			// damn hacks for correct response to filelist request
-			SendADCSnd( "file", adcTTH, pos, m_nChunkSize );
+			SendADCSnd( "file", adcTTH, pos, m_nChunkSize, compress );
 		}
 		else if (adcTTH == "") // ADCGet without TTH
 		{
-			SendADCSnd( "file", "/" + dstfile, pos, m_nChunkSize );
+			// escape spaces
+			dstfile = dstfile.Replace(" ", "\\ ");
+			
+			SendADCSnd( "file", "/" + dstfile, pos, m_nChunkSize, compress );
 		}
 		else
 		{
-			SendADCSnd( "file", "TTH/" + adcTTH, pos, m_nChunkSize );
+			SendADCSnd( "file", "TTH/" + adcTTH, pos, m_nChunkSize, compress );
 		}
 		
 		if ( (eMode != estTRANSFERUPLOAD) || (m_bIdle == FALSE) )
@@ -546,7 +565,7 @@ int CTransfer::StartUpload( CString dstfile, ulonglong length, ulonglong pos, ul
 			DoInitUpload();
 		}
 	}
-	else if (uget)
+	else if ( (uget) || (compress) )
 	{
 		SendSending(m_nChunkSize);
 		
@@ -941,6 +960,12 @@ int CTransfer::HandleMessage( char * c, int )
 							s += "TTHL ";
 						}
 
+						if ( CConfig::Instance()->GetCompressedTransfers() == TRUE )
+						{
+							s += "GetZBlock ";
+							s += "ZLIG ";
+						}
+
 						if ( s != "" )
 						{
 							SendSupports(s);
@@ -1102,7 +1127,7 @@ void CTransfer::DataSend()
 			}
 		}
 
-		freebuf = READ_FILE_BUFFER_SIZE; //GetFreeSendBufferSize();
+		freebuf = m_nReadFileBufferSize; //GetFreeSendBufferSize();
 		
 		// should we test for zblock also?
 		if ( ((m_MessageSupports.m_bChunk == FALSE) && (m_MessageSupports.m_bADCGet == FALSE) && (m_MessageSupports.m_bXMLBZList == FALSE) && (m_MessageSupports.m_bZBlock == FALSE)) || (m_nChunkSize == 0) )
@@ -1121,10 +1146,130 @@ void CTransfer::DataSend()
 			// load data from filehandle and send ...
 			if ( m_nFileBufferPos == m_nFileBufferSize )
 			{
-				if ( len > READ_FILE_BUFFER_SIZE )
+				if ( len > m_nReadFileBufferSize )
 				{
-					len = READ_FILE_BUFFER_SIZE;
+					len = m_nReadFileBufferSize;
 				}
+				
+				if ( m_nDataType == 2 )
+				{
+					pByteArray->SetSize(0);
+					m_nFileBufferSize = 0;
+					
+					CByteArray readbuffer;
+					
+					long fileBytesRead = 0;
+					
+					while ( pByteArray->Size() < (len / 2) )
+					{
+						bool more = TRUE;
+						int inlen = 0;
+						int outlen = 0;
+						bool res = TRUE;
+						CByteArray temp;
+						
+						if ( m_nTransfered + fileBytesRead >= m_nChunkSize )
+						{
+							//printf("No more data ");
+							more = FALSE;
+							
+							// this just causes a world of pain
+							/* res = TRUE;
+							
+							while ( res )
+							{
+								temp.SetSize( m_nReadFileBufferSize );
+								inlen = 0;
+								outlen = temp.Size();
+								
+								res = m_Deflater.DeflateBlock( (char *)0, &inlen, (char *)temp.Data(), &outlen, more );
+								
+								if ( outlen > 0 );
+								{
+									printf(" got %d ", outlen);
+									pByteArray->Append( temp.Data(), outlen );
+									m_nFileBufferSize += outlen;
+								}
+							} */
+							
+							//printf("DeflateEnd\n");
+							break;
+						}
+						else
+						{
+							// limit to reading only up to end of this chunk
+							if ( (m_nTransfered + fileBytesRead + len) > m_nChunkSize )
+							{
+								len = m_nChunkSize - (m_nTransfered + fileBytesRead);
+								//printf("Limiting len to %llu\n", len);
+							}
+							
+							readbuffer.SetSize(len);
+						
+							long read = m_File.Read( (char *)readbuffer.Data(), (long)len );
+						
+							if ( read == -1 )
+							{
+								CallBack_SendError(strerror(errno));
+								perror("CTransfer::DataSend() read error ! ");
+								Disconnect(TRUE);
+								return;
+							}
+							else if ( read == 0 )
+							{
+								perror("CTransfer::DataSend() no data read ! ");
+								Disconnect(TRUE);
+								return;
+							}
+							else
+							{
+								fileBytesRead += read;
+								
+								if ( m_nTransfered + fileBytesRead >= m_nChunkSize )
+								{
+									more = FALSE;
+								}
+								
+								temp.SetSize( m_nReadFileBufferSize * 2 );
+							
+								inlen = read;
+								outlen = temp.Size();
+							
+								res = m_Deflater.DeflateBlock( (char *)readbuffer.Data(), &inlen, (char *)temp.Data(), &outlen, more );
+								
+								if ( (res == TRUE) || (more == FALSE) )
+								{
+									if ( outlen > 0 )
+									{
+										pByteArray->Append( temp.Data(), outlen );
+										m_nFileBufferSize += outlen;
+									}
+								}
+								else
+								{
+									CallBack_SendError("Zlib compression failed");
+									printf("CTransfer::DataSend() deflate failed\n");
+									Disconnect(TRUE);
+									return;
+								}
+							}
+						}
+					}
+					
+					m_nFileBufferPos = 0;
+					len = m_nFileBufferSize;
+					
+					// must either change sizes or adjust m_nTransfered
+					//SetLength( GetLength() + m_nFileBufferSize - fileBytesRead );
+					//SetEndPosition( GetEndPosition() + m_nFileBufferSize - fileBytesRead );
+					//m_nChunkSize = m_nChunkSize + m_nFileBufferSize - fileBytesRead;
+					
+					m_nTransfered = m_nTransfered + fileBytesRead - m_nFileBufferSize;
+					
+					//printf( "Compressed: %ld to %ld\n", fileBytesRead, m_nFileBufferSize );
+				}
+				else // m_nDataType == 2
+				{
 
 				m_nFileBufferSize = m_File.Read( (char *)pByteArray->Data(), (long)len );
 
@@ -1148,6 +1293,8 @@ void CTransfer::DataSend()
 				}
 
 				m_nFileBufferPos = 0;
+				
+				} // m_nDataType == 2
 			}
 
 			// limit to sendbuffer
@@ -1727,10 +1874,10 @@ void CTransfer::DoInitUpload()
 	// init filehandle
 	if ( eMedium == eltFILE )
 	{
-		m_nFileBufferPos    = READ_FILE_BUFFER_SIZE;
-		m_nFileBufferSize   = READ_FILE_BUFFER_SIZE;
+		m_nFileBufferPos    = m_nReadFileBufferSize;
+		m_nFileBufferSize   = m_nReadFileBufferSize;
 		m_nPendingSendBytes = 0;
-		pByteArray->SetSize(READ_FILE_BUFFER_SIZE);
+		pByteArray->SetSize(m_nReadFileBufferSize);
 		m_File.Close();
 		
 		if ( m_File.Open( sSrcFilename, IO_RAW | IO_READONLY ) == FALSE )
@@ -1761,6 +1908,29 @@ void CTransfer::DoInitUpload()
 	}
 	else
 	{
+		// compress buffer if necessary
+		if ( m_nDataType == 2 )
+		{
+			CByteArray * compressed = new CByteArray();
+			unsigned long outlen = m_Deflater.OneShotDeflate( (const char*)pByteArray->Data(), pByteArray->Size(), compressed );
+			if ( outlen > 0 )
+			{
+				//printf("Compressed buffer from %ld to %ld\n", pByteArray->Size(), outlen );
+				pByteArray->SetSize(0);
+				pByteArray->Append( compressed->Data(), outlen );
+				
+				// hack
+				SetEndPosition(outlen);
+				SetLength(outlen);
+				m_nChunkSize = outlen;
+			}
+			else
+			{
+				printf("CTransfer::DoInitUpload: Compress buffer failed!\n");
+			}
+			delete compressed;
+		}
+		
 		// start transfer
 		m_bIdle = FALSE;
 	}
