@@ -82,7 +82,7 @@ public:
 	/** */
 	int StartDownload( CString dstfile, ulonglong startposition, ulonglong endposition, ulonglong size, ulonglong chunksize, CString srcfile, CString adcTTH = "" );
 	/** */
-	int StartUpload( CString dstfile, ulonglong length, ulonglong pos, ulonglong chunksize, CString srcfile, bool uget = false, bool adcget = false, CString adcTTH = "" );
+	int StartUpload( CString dstfile, ulonglong length, ulonglong pos, ulonglong chunksize, CString srcfile, bool uget = false, bool adcget = false, CString adcTTH = "", bool compress = false );
 
 	/** */
 	int CallBack_SendObject( CObject * Object );
@@ -291,6 +291,10 @@ private:
 	struct sAverageTransferRate m_AverageTransferRate;
 	/** */
 	CZLib m_ZLib;
+	/** used to compress uploads */
+	CDeflater m_Deflater;
+	/** size of read buffer */
+	ulonglong m_nReadFileBufferSize;
 };
 
 /** */
