@@ -125,7 +125,11 @@ typedef enum eDCMessage {
 	//DC_MESSAGE_FAILED
 	// ADCGET
 	DC_MESSAGE_ADCGET,
-	DC_MESSAGE_ADCSND
+	DC_MESSAGE_ADCSND,
+	// UserIP2 message
+	DC_MESSAGE_USERIP,
+	// ZLine message
+	DC_MESSAGE_ZLINE
 };
 
 #define EDONKEY_HASH_BLOCK_SIZE 9728000
@@ -337,6 +341,7 @@ public:
 		m_bClientID  = FALSE;
 		m_bSecExec   = FALSE;
 		m_bFeed      = FALSE;
+		m_bZLine     = FALSE;
 	};
 	/** */
 	virtual ~CMessageSupports() {};
@@ -370,6 +375,7 @@ public:
 		m_bClientID  = p.m_bClientID;
 		m_bSecExec   = p.m_bSecExec;
 		m_bFeed      = p.m_bFeed;
+		m_bZLine     = p.m_bZLine;
 		return *this;
 	};
 
@@ -421,6 +427,8 @@ public:
 	bool m_bSecExec;
 	/** */
 	bool m_bFeed;
+	/** */
+	bool m_bZLine;
 };
 
 class DLL_EXPORT CMessageCapabilities : public CDCMessage {
@@ -482,6 +490,7 @@ public:
 		m_nPos  = 0;
 		m_nSize = 0;
 		m_bUGet = false;
+		m_bZLib = false;
 	};
 	/** */
 	virtual ~CMessageGet() {};
@@ -495,6 +504,7 @@ public:
 		m_nPos      = p.m_nPos;
 		m_nSize     = p.m_nSize;
 		m_bUGet     = p.m_bUGet;
+		m_bZLib	    = p.m_bZLib;
 
 		return *this;
 	};
@@ -507,6 +517,8 @@ public:
 	ulonglong m_nSize;
 	/** True if remote did $UGetBlock, false if $Get was used */
 	bool m_bUGet;
+	/** True if remote did $GetZBlock / $UGetZBlock */
+	bool m_bZLib;
 };
 
 class DLL_EXPORT CMessageADCGet : public CDCMessage {
@@ -1675,4 +1687,27 @@ public:
 	CString m_sHubHost;
 };
 
+class DLL_EXPORT CMessageUserIP : public CDCMessage {
+public:
+	/** Creates a new UserIP object */
+	CMessageUserIP() : CDCMessage(DC_MESSAGE_USERIP) {
+	}
+	/** */
+	virtual ~CMessageUserIP() {};
+	
+	/** Nick */
+	CString m_sNick;
+	/** IP address */
+	CString m_sIP;
+};
+
+class DLL_EXPORT CMessageZLine : public CDCMessage {
+public:
+	/** */
+	CMessageZLine() : CDCMessage(DC_MESSAGE_ZLINE) {
+	}
+	/** */
+	virtual ~CMessageZLine() {};
+};
+
 #endif
